Package: fishpond
Title: Fishpond: downstream methods and tools for expression data
Version: 2.18.0
Authors@R: c(
    person("Anqi", "Zhu", role=c("aut","ctb")),
    person("Michael", "Love", email="michaelisaiahlove@gmail.com", role=c("aut","cre")),
    person("Avi", "Srivastava", role=c("aut","ctb")),
    person("Rob", "Patro", role=c("aut","ctb")),
    person("Joseph", "Ibrahim", role=c("aut","ctb")),
    person("Hirak", "Sarkar", role="ctb"),
    person("Euphy", "Wu", role="ctb"),
    person("Noor", "Pratap Singh", role="ctb"),
    person("Scott", "Van Buren", role="ctb"),
    person("Dongze", "He", role="ctb"),
    person("Steve", "Lianoglou", role="ctb"),
    person("Wes", "Wilson", role="ctb"),
    person("Jeroen", "Gilis", role="ctb")
    )
Maintainer: Michael Love <michaelisaiahlove@gmail.com>
Description: Fishpond contains methods for differential transcript
    and gene expression analysis of RNA-seq data using inferential
    replicates for uncertainty of abundance quantification,
    as generated by Gibbs sampling or bootstrap sampling. Also the
    package contains a number of utilities for working with Salmon
    and Alevin quantification files.
Imports: graphics, stats, utils, methods, abind, gtools, qvalue,
        S4Vectors, IRanges, SummarizedExperiment, GenomicRanges,
        matrixStats, svMisc, Matrix, SingleCellExperiment, jsonlite
Suggests: testthat, knitr, rmarkdown, macrophage, tximeta,
        org.Hs.eg.db, samr, DESeq2, apeglm, tximportData, limma,
        ensembldb, EnsDb.Hsapiens.v86, GenomicFeatures, AnnotationDbi,
        pheatmap, Gviz, GenomeInfoDb, data.table
License: GPL-2
Encoding: UTF-8
URL: https://thelovelab.github.io/fishpond,
        https://thelovelab.com/mikelove/fishpond
BugReports: https://support.bioconductor.org/tag/fishpond
biocViews: Sequencing, RNASeq, GeneExpression, Transcription,
        Normalization, Regression, MultipleComparison, BatchEffect,
        Visualization, DifferentialExpression, DifferentialSplicing,
        AlternativeSplicing, SingleCell
VignetteBuilder: knitr
RoxygenNote: 7.2.3
git_url: https://git.bioconductor.org/packages/fishpond
git_branch: RELEASE_3_23
git_last_commit: d572667
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:08:25 UTC; biocbuild
Author: Anqi Zhu [aut, ctb],
  Michael Love [aut, cre],
  Avi Srivastava [aut, ctb],
  Rob Patro [aut, ctb],
  Joseph Ibrahim [aut, ctb],
  Hirak Sarkar [ctb],
  Euphy Wu [ctb],
  Noor Pratap Singh [ctb],
  Scott Van Buren [ctb],
  Dongze He [ctb],
  Steve Lianoglou [ctb],
  Wes Wilson [ctb],
  Jeroen Gilis [ctb]
