# Generated by roxygen2: do not edit by hand

export(addStatsFromCSV)
export(alevinEC)
export(computeInfRV)
export(deswish)
export(getTrace)
export(importAllelicCounts)
export(isoformProportions)
export(labelKeep)
export(loadFry)
export(makeInfReps)
export(makeSimSwishData)
export(makeTx2Tss)
export(miniSwish)
export(plotAllelicGene)
export(plotAllelicHeatmap)
export(plotInfReps)
export(plotMASwish)
export(salmonEC)
export(scaleInfReps)
export(splitSwish)
export(swish)
importFrom(GenomicRanges,"end<-")
importFrom(GenomicRanges,"start<-")
importFrom(GenomicRanges,"strand<-")
importFrom(GenomicRanges,end)
importFrom(GenomicRanges,flank)
importFrom(GenomicRanges,resize)
importFrom(GenomicRanges,seqnames)
importFrom(GenomicRanges,sort)
importFrom(GenomicRanges,start)
importFrom(GenomicRanges,strand)
importFrom(GenomicRanges,width)
importFrom(IRanges,CharacterList)
importFrom(IRanges,IntegerList)
importFrom(Matrix,colSums)
importFrom(Matrix,readMM)
importFrom(Matrix,rowSums)
importFrom(Matrix,sparseMatrix)
importFrom(Matrix,sparseVector)
importFrom(Matrix,t)
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,metadata)
importFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(SummarizedExperiment,"assay<-")
importFrom(SummarizedExperiment,"assayNames<-")
importFrom(SummarizedExperiment,"assays<-")
importFrom(SummarizedExperiment,"colData<-")
importFrom(SummarizedExperiment,"mcols<-")
importFrom(SummarizedExperiment,"rowRanges<-")
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assayNames)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,mcols)
importFrom(SummarizedExperiment,rowRanges)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,legend)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,rect)
importFrom(graphics,segments)
importFrom(graphics,title)
importFrom(gtools,permutations)
importFrom(jsonlite,fromJSON)
importFrom(matrixStats,colVars)
importFrom(matrixStats,rowMedians)
importFrom(matrixStats,rowQuantiles)
importFrom(matrixStats,rowRanks)
importFrom(matrixStats,rowVars)
importFrom(methods,as)
importFrom(methods,is)
importFrom(methods,slot)
importFrom(stats,cor)
importFrom(stats,median)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rnbinom)
importFrom(stats,rpois)
importFrom(stats,runif)
importFrom(stats,var)
importFrom(svMisc,progress)
importFrom(utils,capture.output)
importFrom(utils,head)
importFrom(utils,read.csv)
importFrom(utils,read.table)
importFrom(utils,tail)
importFrom(utils,write.table)
